// Defined to disable deprecated functions warning in Visual C++ 2005
#define _CRT_SECURE_NO_DEPRECATE 1

#include <windows.h>
#include <stdio.h>


#pragma pack(1)       // set byte packing
typedef struct {
	int bLength;          // Length of structure in bytes
	int iRefLevel;        // AGC reference level in dB
	DWORD dwAttackTime;   // AGC attack time in ms
	DWORD dwDecayTime;    // AGC decay time in ms
} SOFTAGC_DATA;
#pragma pack()       // set back the default packing

HINSTANCE WRG315API=NULL;
DWORD WRG315API_Count=0;

typedef void (__stdcall *FNCDemodCallback)(void *Target,void *Buf,DWORD Size,DWORD SamplingRate);

typedef int (__stdcall *FNCOpenRadioDevice)(int iDeviceNum);
typedef BOOL (__stdcall *FNCCloseRadioDevice)(int hRadio);
typedef BOOL (__stdcall *FNCInitializeDemodulator)(int hRadio,char *CalFilePath);
typedef BOOL (__stdcall *FNCSetAGC)(int hRadio,DWORD iAGC);
typedef BOOL (__stdcall *FNCSetFrequency)(int hRadio,DWORD iFreq);
typedef BOOL (__stdcall *FNCSetPower)(int hRadio,BOOL iFreq);
typedef BOOL (__stdcall *FNCSetAudioBandwidth)(int hRadio,DWORD BW);
typedef BOOL (__stdcall *FNCSetIFBandwidth)(int hRadio,DWORD BW);
typedef BOOL (__stdcall *FNCSetMode)(int hRadio,DWORD Mode);
typedef BOOL (__stdcall *FNCSetSoftAGC)(int hRadio,SOFTAGC_DATA *Data);
typedef BOOL (__stdcall *FNCSetVolume)(int hRadio,DWORD AudioGain);
typedef BOOL (__stdcall *FNCSetupStreams)(int hRadio,FNCDemodCallback IFH,void *IFT,FNCDemodCallback AudioH,void *AudioT);

FNCOpenRadioDevice OpenRadioDevice=NULL;
FNCCloseRadioDevice CloseRadioDevice=NULL;
FNCInitializeDemodulator InitializeDemodulator=NULL;
FNCSetAGC SetAGC=NULL;
FNCSetFrequency SetFrequency=NULL;
FNCSetPower SetPower=NULL;
FNCSetAudioBandwidth SetAudioBandwidth=NULL;
FNCSetIFBandwidth SetIFBandwidth=NULL;
FNCSetMode SetMode=NULL;
FNCSetSoftAGC SetSoftAGC=NULL;
FNCSetVolume SetVolume=NULL;
FNCSetupStreams SetupStreams=NULL;

void __stdcall APIUnload(void)
{
	//  If the API has not been loaded it simply returns. Else if the API has been
	// loaded more than once, the special counter is decremented. Only if the
	// special counter drops back to 0 the library is actually freed.
	if (!WRG315API_Count || --WRG315API_Count!=0) return;
	FreeLibrary(WRG315API);
	WRG315API=NULL;
	CloseRadioDevice=NULL;
	OpenRadioDevice=NULL;
	InitializeDemodulator=NULL;
	SetAGC=NULL;
	SetFrequency=NULL;
	SetPower=NULL;
	SetAudioBandwidth=NULL;
	SetIFBandwidth=NULL;
	SetMode=NULL;
	SetSoftAGC=NULL;
	SetVolume=NULL;
	SetupStreams=NULL;
}

BOOL __stdcall APILoad(void)
{
	//  If the API has already been loaded, only increment the special counter and
	// return with success.
	if (WRG315API_Count) { WRG315API_Count++;return TRUE; }
	//  Tries to load the API. If this fails it returns with error.
	if ((WRG315API=LoadLibrary("WRG315API"))==NULL) return FALSE;
	//  The API has been successfully loaded. Now it has to connect to its
	// exported functions.
	CloseRadioDevice=(FNCCloseRadioDevice)GetProcAddress(WRG315API,"CloseRadioDevice");
	OpenRadioDevice=(FNCOpenRadioDevice)GetProcAddress(WRG315API,"OpenRadioDevice");
	InitializeDemodulator=(FNCInitializeDemodulator)GetProcAddress(WRG315API,"InitializeDemodulator");
	SetAGC=(FNCSetAGC)GetProcAddress(WRG315API,"SetAGC");
	SetFrequency=(FNCSetFrequency)GetProcAddress(WRG315API,"SetFrequency");
	SetPower=(FNCSetPower)GetProcAddress(WRG315API,"SetPower");
	SetAudioBandwidth=(FNCSetAudioBandwidth)GetProcAddress(WRG315API,"SetAudioBandwidth");
	SetIFBandwidth=(FNCSetIFBandwidth)GetProcAddress(WRG315API,"SetIFBandwidth");
	SetMode=(FNCSetMode)GetProcAddress(WRG315API,"SetMode");
	SetSoftAGC=(FNCSetSoftAGC)GetProcAddress(WRG315API,"SetSoftAGC");
	SetVolume=(FNCSetVolume)GetProcAddress(WRG315API,"SetVolume");
	SetupStreams=(FNCSetupStreams)GetProcAddress(WRG315API,"SetupStreams");
	//  If all functions have been found it increments the special counter. If not
	// the library is unloaded.
	WRG315API_Count++;
	if (CloseRadioDevice && OpenRadioDevice && InitializeDemodulator && SetAGC && SetFrequency && SetPower && 
		SetAudioBandwidth && SetIFBandwidth && SetMode && SetSoftAGC && SetVolume && SetupStreams)
		return TRUE;
	//  As the library has not been completely loaded, it clears any already
	// initialized data.
	APIUnload();
	return FALSE;
}

#define RXAGC_FAST            3
#define RADIOMODE_AM          1

void __stdcall SamplesIFHandler(void *Target,void *Buf,DWORD Size,DWORD SamplingRate)
{
	if (Target) fwrite(Buf,1,Size,(FILE *)Target);
}
void __stdcall SamplesAudioHandler(void *Target,void *Buf,DWORD Size,DWORD SamplingRate)
{
	if (Target) fwrite(Buf,1,Size,(FILE *)Target);
}

int main(int argc, char* argv[])
{
	// load the G3 API library
	if (!APILoad()) {
		puts("WRG315API.DLL cannot be loaded!");
		return 0;
	}

	// open the first available radio
	int hRadio=OpenRadioDevice(0);
	if (!hRadio) {
		puts("No WR-G315 device could be opened !");
		APIUnload();
		return 0;
	}

	// configure the RF hardware
	SetPower(hRadio,TRUE);
	SetAGC(hRadio,3);
	SetFrequency(hRadio,855000);

	// start the advanced mode
	SOFTAGC_DATA AGCData = { sizeof(SOFTAGC_DATA), -6, 5, 200 };

	puts("Initializing demodulator..");
	InitializeDemodulator(hRadio,NULL);
	SetIFBandwidth(hRadio,10000);
	SetSoftAGC(hRadio,&AGCData);
	SetMode(hRadio,RADIOMODE_AM);
	SetAudioBandwidth(hRadio,5000);
	SetVolume(hRadio,20);

	// configure the streams
	FILE *Audio,*IF;

	Audio=fopen("Audio.pcm","wb");
	IF=fopen("IF.pcm","wb");
	puts("Starting streaming..");
	SetupStreams(hRadio,SamplesIFHandler,IF,SamplesAudioHandler,Audio);

	//
	// main application body comes here, the following instructions just waits for a key to be pressed
	//
	puts("Press ENTER key to stop recording..");
	getc(stdin);

	// stop the streams
	SetupStreams(hRadio,NULL,NULL,NULL,NULL);
	// close the samples files
	if (Audio) fclose(Audio);
	if (IF) fclose(IF);

	// close the device handle
	if (CloseRadioDevice(hRadio)) puts("The device is closed properly");
	else puts("The device failed to close");

	// free the G3 API library
	APIUnload();
	return 0;
}
